@@ -14,7 +14,8 @@
 
 import copy
 import os.path
-import urllib
+import typing as ty
+from urllib import parse
 import warnings
 
 try:
@@ -24,23 +25,25 @@ except ImportError:
 
 from keystoneauth1 import discover
 import keystoneauth1.exceptions.catalog
+from keystoneauth1.identity import base as ks_identity_base
 from keystoneauth1.loading import adapter as ks_load_adap
+from keystoneauth1 import plugin
 from keystoneauth1 import session as ks_session
 import os_service_types
 import requestsexceptions
 
 try:
-    import statsd
+    import statsd as statsd_client
 except ImportError:
-    statsd = None
+    statsd_client = None
 try:
     import prometheus_client
 except ImportError:
     prometheus_client = None
 try:
-    import influxdb
+    import influxdb as influxdb_client
 except ImportError:
-    influxdb = None
+    influxdb_client = None
 
 from openstack import _log
 from openstack.config import _util
@@ -50,6 +53,11 @@ from openstack import proxy
 from openstack import version as openstack_version
 from openstack import warnings as os_warnings
 
+if ty.TYPE_CHECKING:
+    from oslo_config import cfg
+
+    from openstack.config import loader
+
 
 _logger = _log.setup_logging('openstack')
 
@@ -65,6 +73,10 @@ SCOPE_KEYS = {
 _ENOENT = object()
 
 
+class _PasswordCallback(ty.Protocol):
+    def __call__(self, prompt: ty.Optional[str] = None) -> str: ...
+
+
 def _make_key(key, service_type):
     if not service_type:
         return key
@@ -75,7 +87,7 @@ def _make_key(key, service_type):
 
 def _disable_service(config, service_type, reason=None):
     service_type = service_type.lower().replace('-', '_')
-    key = 'has_{service_type}'.format(service_type=service_type)
+    key = f'has_{service_type}'
     config[key] = False
     if reason:
         d_key = _make_key('disabled_reason', service_type)
@@ -95,14 +107,14 @@ def _get_implied_microversion(version):
 
 
 def from_session(
-    session,
-    name=None,
-    region_name=None,
-    force_ipv4=False,
-    app_name=None,
-    app_version=None,
-    **kwargs,
-):
+    session: ks_session.Session,
+    name: ty.Optional[str] = None,
+    region_name: ty.Optional[str] = None,
+    force_ipv4: bool = False,
+    app_name: ty.Optional[str] = None,
+    app_version: ty.Optional[str] = None,
+    **kwargs: ty.Any,
+) -> 'CloudRegion':
     """Construct a CloudRegion from an existing `keystoneauth1.session.Session`
 
     When a Session already exists, we don't actually even need to go through
@@ -138,7 +150,12 @@ def from_session(
     )
 
 
-def from_conf(conf, session=None, service_types=None, **kwargs):
+def from_conf(
+    conf: 'cfg.ConfigOpts',
+    session: ty.Optional[ks_session.Session] = None,
+    service_types: ty.Optional[list[str]] = None,
+    **kwargs: ty.Any,
+) -> 'CloudRegion':
     """Create a CloudRegion from oslo.config ConfigOpts.
 
     :param oslo_config.cfg.ConfigOpts conf:
@@ -189,13 +206,11 @@ def from_conf(conf, session=None, service_types=None, **kwargs):
                 _disable_service(
                     config_dict,
                     st,
-                    reason="No section for project '{project}' (service type "
-                    "'{service_type}') was present in the config.".format(
-                        project=project_name, service_type=st
-                    ),
+                    reason=f"No section for project '{project_name}' (service type "
+                    f"'{st}') was present in the config.",
                 )
                 continue
-        opt_dict = {}
+        opt_dict: dict[str, str] = {}
         # Populate opt_dict with (appropriately processed) Adapter conf opts
         try:
             ks_load_adap.process_conf_options(conf[project_name], opt_dict)
@@ -211,15 +226,10 @@ def from_conf(conf, session=None, service_types=None, **kwargs):
             # *that* blow up.
             reason = (
                 "Encountered an exception attempting to process config "
-                "for project '{project}' (service type "
-                "'{service_type}'): {exception}".format(
-                    project=project_name, service_type=st, exception=e
-                )
-            )
-            _logger.warning(
-                "Disabling service '{service_type}': "
-                "{reason}".format(service_type=st, reason=reason)
+                f"for project '{project_name}' (service type "
+                f"'{st}'): {e}"
             )
+            _logger.warning(f"Disabling service '{st}': {reason}")
             _disable_service(config_dict, st, reason=reason)
             continue
         # Load them into config_dict under keys prefixed by ${service_type}_
@@ -236,12 +246,13 @@ class CloudRegion:
     A CloudRegion encapsulates the config information needed for connections
     to all of the services in a Region of a Cloud.
 
-    :param str region_name:
+    :param name:
+    :param region_name:
         The default region name for all services in this CloudRegion. If
-        both ``region_name`` and ``config['region_name'] are specified, the
+        both ``region_name`` and ``config['region_name']`` are specified, the
         kwarg takes precedence. May be overridden for a given ${service}
         via a ${service}_region_name key in the ``config`` dict.
-    :param dict config:
+    :param config:
         A dict of configuration values for the CloudRegion and its
         services. The key for a ${config_option} for a specific ${service}
         should be ${service}_${config_option}. For example, to configure
@@ -255,35 +266,59 @@ class CloudRegion:
         key, e.g.::
 
             'interface': 'public'
+    :param force_ipv4:
+    :param auth_plugin:
+    :param openstack_config:
+    :param session_constructor:
+    :param app_name:
+    :param app_version:
+    :param session:
+    :param discovery_cache:
+    :param extra_config:
+    :param cache_expiration_time:
+    :param cache_expirations:
+    :param cache_path:
+    :param cache_class:
+    :param cache_arguments:
+    :param password_callback:
+    :param statsd_host:
+    :param statsd_port:
+    :param statsd_prefix:
+    :param influxdb_config:
+    :param collector_registry:
+    :param cache_auth:
     """
 
     def __init__(
         self,
-        name=None,
-        region_name=None,
-        config=None,
-        force_ipv4=False,
-        auth_plugin=None,
-        openstack_config=None,
-        session_constructor=None,
-        app_name=None,
-        app_version=None,
-        session=None,
-        discovery_cache=None,
-        extra_config=None,
-        cache_expiration_time=0,
-        cache_expirations=None,
-        cache_path=None,
-        cache_class='dogpile.cache.null',
-        cache_arguments=None,
-        password_callback=None,
-        statsd_host=None,
-        statsd_port=None,
-        statsd_prefix=None,
-        influxdb_config=None,
-        collector_registry=None,
-        cache_auth=False,
-    ):
+        name: ty.Optional[str] = None,
+        region_name: ty.Optional[str] = None,
+        config: ty.Optional[dict[str, ty.Any]] = None,
+        force_ipv4: bool = False,
+        auth_plugin: ty.Optional[plugin.BaseAuthPlugin] = None,
+        openstack_config: ty.Optional['loader.OpenStackConfig'] = None,
+        session_constructor: ty.Optional[type[ks_session.Session]] = None,
+        app_name: ty.Optional[str] = None,
+        app_version: ty.Optional[str] = None,
+        session: ty.Optional[ks_session.Session] = None,
+        discovery_cache: ty.Optional[dict[str, discover.Discover]] = None,
+        extra_config: ty.Optional[dict[str, ty.Any]] = None,
+        cache_expiration_time: int = 0,
+        cache_expirations: ty.Optional[dict[str, int]] = None,
+        cache_path: ty.Optional[str] = None,
+        cache_class: str = 'dogpile.cache.null',
+        cache_arguments: ty.Optional[dict[str, ty.Any]] = None,
+        password_callback: ty.Optional[_PasswordCallback] = None,
+        statsd_host: ty.Optional[str] = None,
+        statsd_port: ty.Optional[str] = None,
+        statsd_prefix: ty.Optional[str] = None,
+        # TODO(stephenfin): Add better types
+        influxdb_config: ty.Optional[dict[str, ty.Any]] = None,
+        collector_registry: ty.Optional[
+            'prometheus_client.CollectorRegistry'
+        ] = None,
+        cache_auth: bool = False,
+    ) -> None:
         self._name = name
         self.config = _util.normalize_keys(config)
         # NOTE(efried): For backward compatibility: a) continue to accept the
@@ -341,13 +376,20 @@ class CloudRegion:
 
     @property
     def name(self):
-        if self._name is None:
-            try:
-                self._name = urllib.parse.urlparse(
-                    self.get_session().auth.auth_url
-                ).hostname
-            except Exception:
-                self._name = self._app_name or ''
+        if self._name is not None:
+            return self._name
+
+        auth = self.get_session().auth
+        # not all auth plugins are identity plugins
+        if (
+            auth
+            and isinstance(auth, ks_identity_base.BaseIdentityPlugin)
+            and auth.auth_url
+        ):
+            self._name = parse.urlparse(auth.auth_url).hostname
+        else:
+            self._name = self._app_name or ''
+
         return self._name
 
     @property
@@ -535,16 +577,18 @@ class CloudRegion:
     def get_service_name(self, service_type):
         return self._get_config('service_name', service_type)
 
-    def get_endpoint(self, service_type):
+    def get_endpoint(self, service_type: str) -> ty.Optional[str]:
         auth = self.config.get('auth', {})
         value = self._get_config('endpoint_override', service_type)
         if not value:
             value = self._get_config('endpoint', service_type)
+
         if not value and self.config.get('auth_type') == 'none':
             # If endpoint is given and we're using the none auth type,
             # then the endpoint value is the endpoint_override for every
             # service.
             value = auth.get('endpoint')
+
         if (
             not value
             and service_type == 'identity'
@@ -554,6 +598,7 @@ class CloudRegion:
             # Specifically, looking up a list of projects/domains/system to
             # scope to.
             value = auth.get('auth_url')
+
         # Because of course. Seriously.
         # We have to override the Rackspace block-storage endpoint because
         # only v1 is in the catalog but the service actually does support
@@ -567,11 +612,15 @@ class CloudRegion:
             and service_type == 'block-storage'
         ):
             value = value + auth.get('project_id')
-        return value
+
+        return str(value) if value else None
 
     def get_endpoint_from_catalog(
-        self, service_type, interface=None, region_name=None
-    ):
+        self,
+        service_type: str,
+        interface: ty.Optional[str] = None,
+        region_name: ty.Optional[str] = None,
+    ) -> ty.Optional[str]:
         """Return the endpoint for a given service as found in the catalog.
 
         For values respecting endpoint overrides, see
@@ -590,14 +639,20 @@ class CloudRegion:
         interface = interface or self.get_interface(service_type)
         region_name = region_name or self.get_region_name(service_type)
         session = self.get_session()
-        catalog = session.auth.get_access(session).service_catalog
+
+        auth = session.auth
+        if not isinstance(auth, ks_identity_base.BaseIdentityPlugin):
+            return None
+
+        catalog = auth.get_access(session).service_catalog
         try:
+            # FIXME(stephenfin): Remove once keystoneauth1 has type hints
             return catalog.url_for(
                 service_type=service_type,
                 interface=interface,
                 region_name=region_name,
             )
-        except keystoneauth1.exceptions.catalog.EndpointNotFound:
+        except (keystoneauth1.exceptions.catalog.EndpointNotFound, ValueError):
             return None
 
     def get_connect_retries(self, service_type):
@@ -635,6 +690,8 @@ class CloudRegion:
         if self.skip_auth_cache():
             return
 
+        assert self._auth is not None  # narrow type
+
         cache_id = self._auth.get_cache_id()
 
         # skip if the plugin does not support caching
@@ -650,18 +707,20 @@ class CloudRegion:
         self.log.debug('Reusing authentication from keyring')
         self._auth.set_auth_state(state)
 
-    def set_auth_cache(self):
+    def set_auth_cache(self) -> None:
         if self.skip_auth_cache():
             return
 
+        assert self._auth is not None  # narrow type
+
         cache_id = self._auth.get_cache_id()
         state = self._auth.get_auth_state()
 
         try:
             if state:
-                # NOTE: under some conditions the method may be invoked when auth
-                # is empty. This may lead to exception in the keyring lib, thus do
-                # nothing.
+                # NOTE: under some conditions the method may be invoked when
+                # auth is empty. This may lead to exception in the keyring lib,
+                # thus do nothing.
                 keyring.set_password('openstacksdk', cache_id, state)
         except RuntimeError:  # the fail backend raises this
             self.log.debug('Failed to set auth into keyring')
@@ -680,11 +739,14 @@ class CloudRegion:
         :class:`~openstack.config.cloud_region.CloudRegion` it may be
         desirable.
         """
+        if not self._keystone_session:
+            return
+
         self._keystone_session.additional_user_agent.append(
             ('openstacksdk', openstack_version.__version__)
         )
 
-    def get_session(self):
+    def get_session(self) -> ks_session.Session:
         """Return a keystoneauth session based on the auth credentials."""
         if self._keystone_session is None:
             if not self._auth:
@@ -697,8 +759,8 @@ class CloudRegion:
             # cert verification
             if not verify:
                 self.log.debug(
-                    "Turning off SSL warnings for {full_name}"
-                    " since verify=False".format(full_name=self.full_name)
+                    f"Turning off SSL warnings for {self.full_name} "
+                    f"since verify=False"
                 )
             requestsexceptions.squelch_warnings(insecure_requests=not verify)
             self._keystone_session = self._session_constructor(
@@ -722,6 +784,9 @@ class CloudRegion:
 
     def get_service_catalog(self):
         """Helper method to grab the service catalog."""
+        # not all auth plugins are identity plugins
+        if not isinstance(self._auth, ks_identity_base.BaseIdentityPlugin):
+            return None
         return self._auth.get_access(self.get_session()).service_catalog
 
     def _get_version_request(self, service_type, version):
@@ -762,13 +827,10 @@ class CloudRegion:
             and implied_microversion != default_microversion
         ):
             raise exceptions.ConfigException(
-                "default_microversion of {default_microversion} was given"
-                " for {service_type}, but api_version looks like a"
-                " microversion as well. Please set api_version to just the"
-                " desired major version, or omit default_microversion".format(
-                    default_microversion=default_microversion,
-                    service_type=service_type,
-                )
+                f"default_microversion of {default_microversion} was given "
+                f"for {service_type}, but api_version looks like a "
+                f"microversion as well. Please set api_version to just the "
+                f"desired major version, or omit default_microversion"
             )
         if implied_microversion:
             default_microversion = implied_microversion
@@ -812,7 +874,7 @@ class CloudRegion:
         if not endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
             if not endpoint.endswith('/'):
                 endpoint += '/'
-            endpoint = urllib.parse.urljoin(endpoint, 'v2.0')
+            endpoint = parse.urljoin(endpoint, 'v2.0')
         return endpoint
 
     def get_session_client(
@@ -893,10 +955,10 @@ class CloudRegion:
             ):
                 if self.get_default_microversion(service_type):
                     raise exceptions.ConfigException(
-                        "A default microversion for service {service_type} of"
-                        " {default_microversion} was requested, but the cloud"
-                        " only supports a minimum of {min_microversion} and"
-                        " a maximum of {max_microversion}.".format(
+                        "A default microversion for service {service_type} of "
+                        "{default_microversion} was requested, but the cloud "
+                        "only supports a minimum of {min_microversion} and "
+                        "a maximum of {max_microversion}.".format(
                             service_type=service_type,
                             default_microversion=default_microversion,
                             min_microversion=discover.version_to_string(
@@ -909,17 +971,17 @@ class CloudRegion:
                     )
                 else:
                     raise exceptions.ConfigException(
-                        "A default microversion for service {service_type} of"
-                        " {default_microversion} was requested, but the cloud"
-                        " only supports a minimum of {min_microversion} and"
-                        " a maximum of {max_microversion}. The default"
-                        " microversion was set because a microversion"
-                        " formatted version string, '{api_version}', was"
-                        " passed for the api_version of the service. If it"
-                        " was not intended to set a default microversion"
-                        " please remove anything other than an integer major"
-                        " version from the version setting for"
-                        " the service.".format(
+                        "A default microversion for service {service_type} of "
+                        "{default_microversion} was requested, but the cloud "
+                        "only supports a minimum of {min_microversion} and "
+                        "a maximum of {max_microversion}. The default "
+                        "microversion was set because a microversion "
+                        "formatted version string, '{api_version}', was "
+                        "passed for the api_version of the service. If it "
+                        "was not intended to set a default microversion "
+                        "please remove anything other than an integer major "
+                        "version from the version setting for "
+                        "the service.".format(
                             service_type=service_type,
                             api_version=self.get_api_version(service_type),
                             default_microversion=default_microversion,
@@ -1141,8 +1203,10 @@ class CloudRegion:
             'concurrency', service_type=service_type
         )
 
-    def get_statsd_client(self):
-        if not statsd:
+    def get_statsd_client(
+        self,
+    ) -> ty.Optional['statsd_client.StatsClientBase']:
+        if not statsd_client:
             if self._statsd_host:
                 self.log.warning(
                     'StatsD python library is not available. '
@@ -1156,25 +1220,29 @@ class CloudRegion:
             statsd_args['port'] = self._statsd_port
         if statsd_args:
             try:
-                return statsd.StatsClient(**statsd_args)
+                return statsd_client.StatsClient(**statsd_args)
             except Exception:
                 self.log.warning('Cannot establish connection to statsd')
                 return None
         else:
             return None
 
-    def get_statsd_prefix(self):
+    def get_statsd_prefix(self) -> str:
         return self._statsd_prefix or 'openstack.api'
 
-    def get_prometheus_registry(self):
+    def get_prometheus_registry(
+        self,
+    ) -> ty.Optional['prometheus_client.CollectorRegistry']:
         if not self._collector_registry and prometheus_client:
             self._collector_registry = prometheus_client.REGISTRY
         return self._collector_registry
 
-    def get_prometheus_histogram(self):
+    def get_prometheus_histogram(
+        self,
+    ) -> ty.Optional['prometheus_client.Histogram']:
         registry = self.get_prometheus_registry()
         if not registry or not prometheus_client:
-            return
+            return None
         # We have to hide a reference to the histogram on the registry
         # object, because it's collectors must be singletons for a given
         # registry but register at creation time.
@@ -1194,10 +1262,12 @@ class CloudRegion:
             registry._openstacksdk_histogram = hist
         return hist
 
-    def get_prometheus_counter(self):
+    def get_prometheus_counter(
+        self,
+    ) -> ty.Optional['prometheus_client.Counter']:
         registry = self.get_prometheus_registry()
         if not registry or not prometheus_client:
-            return
+            return None
         counter = getattr(registry, '_openstacksdk_counter', None)
         if not counter:
             counter = prometheus_client.Counter(
@@ -1216,7 +1286,7 @@ class CloudRegion:
 
     def has_service(self, service_type):
         service_type = service_type.lower().replace('-', '_')
-        key = 'has_{service_type}'.format(service_type=service_type)
+        key = f'has_{service_type}'
         return self.config.get(
             key, self._service_type_manager.is_official(service_type)
         )
@@ -1224,9 +1294,9 @@ class CloudRegion:
     def disable_service(self, service_type, reason=None):
         _disable_service(self.config, service_type, reason=reason)
 
-    def enable_service(self, service_type):
+    def enable_service(self, service_type: str) -> None:
         service_type = service_type.lower().replace('-', '_')
-        key = 'has_{service_type}'.format(service_type=service_type)
+        key = f'has_{service_type}'
         self.config[key] = True
 
     def get_disabled_reason(self, service_type):
@@ -1234,8 +1304,11 @@ class CloudRegion:
         d_key = _make_key('disabled_reason', service_type)
         return self.config.get(d_key)
 
-    def get_influxdb_client(self):
-        influx_args = {}
+    def get_influxdb_client(
+        self,
+    ) -> ty.Optional['influxdb_client.InfluxDBClient']:
+        # TODO(stephenfin): We could do with a typed dict here.
+        influx_args: dict[str, ty.Any] = {}
         if not self._influxdb_config:
             return None
         use_udp = bool(self._influxdb_config.get('use_udp', False))
@@ -1250,9 +1323,9 @@ class CloudRegion:
         for key in ['host', 'username', 'password', 'database', 'timeout']:
             if key in self._influxdb_config:
                 influx_args[key] = self._influxdb_config[key]
-        if influxdb and influx_args:
+        if influxdb_client and influx_args:
             try:
-                return influxdb.InfluxDBClient(**influx_args)
+                return influxdb_client.InfluxDBClient(**influx_args)
             except Exception:
                 self.log.warning('Cannot establish connection to InfluxDB')
         else:
