@@ -21,11 +21,13 @@ import json
 import os
 import re
 import sys
+import typing as ty
 import warnings
 
-import appdirs
 from keystoneauth1 import adapter
 from keystoneauth1 import loading
+from keystoneauth1 import session
+import platformdirs
 import yaml
 
 from openstack import _log
@@ -36,9 +38,14 @@ from openstack.config import vendors
 from openstack import exceptions
 from openstack import warnings as os_warnings
 
-APPDIRS = appdirs.AppDirs('openstack', 'OpenStack', multipath='/etc')
-CONFIG_HOME = APPDIRS.user_config_dir
-CACHE_PATH = APPDIRS.user_cache_dir
+if ty.TYPE_CHECKING:
+    from keystoneauth1.loading._plugins.identity import v3 as v3_loaders
+
+PLATFORMDIRS = platformdirs.PlatformDirs(
+    'openstack', 'OpenStack', multipath=True
+)
+CONFIG_HOME = PLATFORMDIRS.user_config_dir
+CACHE_PATH = PLATFORMDIRS.user_cache_dir
 
 # snaps do set $HOME to something like
 # /home/$USER/snap/openstackclients/$SNAP_VERSION
@@ -55,7 +62,7 @@ else:
     )
 UNIX_SITE_CONFIG_HOME = '/etc/openstack'
 
-SITE_CONFIG_HOME = APPDIRS.site_config_dir
+SITE_CONFIG_HOME = PLATFORMDIRS.site_config_dir
 
 CONFIG_SEARCH_PATH = [
     os.getcwd(),
@@ -129,16 +136,16 @@ def _fix_argv(argv):
                 argv[index] = "=".join(split_args)
             # Save both for later so we can throw an error about dupes
             processed[new].add(orig)
-    overlap = []
+    overlap: list[str] = []
     for new, old in processed.items():
         if len(old) > 1:
             overlap.extend(old)
     if overlap:
         raise exceptions.ConfigException(
-            "The following options were given: '{options}' which contain"
-            " duplicates except that one has _ and one has -. There is"
-            " no sane way for us to know what you're doing. Remove the"
-            " duplicate option and try again".format(options=','.join(overlap))
+            "The following options were given: '{options}' which contain "
+            "duplicates except that one has _ and one has -. There is "
+            "no sane way for us to know what you're doing. Remove the "
+            "duplicate option and try again".format(options=','.join(overlap))
         )
 
 
@@ -151,22 +158,22 @@ class OpenStackConfig:
 
     def __init__(
         self,
-        config_files=None,
-        vendor_files=None,
-        override_defaults=None,
-        force_ipv4=None,
-        envvar_prefix=None,
-        secure_files=None,
-        pw_func=None,
-        session_constructor=None,
-        app_name=None,
-        app_version=None,
-        load_yaml_config=True,
-        load_envvars=True,
-        statsd_host=None,
-        statsd_port=None,
-        statsd_prefix=None,
-        influxdb_config=None,
+        config_files: ty.Optional[list[str]] = None,
+        vendor_files: ty.Optional[list[str]] = None,
+        override_defaults: ty.Optional[dict[str, ty.Any]] = None,
+        force_ipv4: ty.Optional[bool] = None,
+        envvar_prefix: ty.Optional[str] = None,
+        secure_files: ty.Optional[list[str]] = None,
+        pw_func: ty.Optional[cloud_region._PasswordCallback] = None,
+        session_constructor: ty.Optional[type[session.Session]] = None,
+        app_name: ty.Optional[str] = None,
+        app_version: ty.Optional[str] = None,
+        load_yaml_config: bool = True,
+        load_envvars: bool = True,
+        statsd_host: ty.Optional[str] = None,
+        statsd_port: ty.Optional[str] = None,
+        statsd_prefix: ty.Optional[str] = None,
+        influxdb_config: ty.Optional[dict[str, ty.Any]] = None,
     ):
         self.log = _log.setup_logging('openstack.config')
         self._session_constructor = session_constructor
@@ -207,15 +214,19 @@ class OpenStackConfig:
 
         # First, use a config file if it exists where expected
         self.config_filename, self.cloud_config = self._load_config_file()
-        _, secure_config = self._load_secure_file()
+        if self.config_filename:
+            self._validate_config_file(self.config_filename, self.cloud_config)
+
+        secure_config_filename, secure_config = self._load_secure_file()
         if secure_config:
+            self._validate_config_file(secure_config_filename, secure_config)
             self.cloud_config = _util.merge_clouds(
                 self.cloud_config, secure_config
             )
 
         if not self.cloud_config:
             self.cloud_config = {'clouds': {}}
-        if 'clouds' not in self.cloud_config:
+        elif 'clouds' not in self.cloud_config:
             self.cloud_config['clouds'] = {}
 
         # Save the other config
@@ -257,12 +268,11 @@ class OpenStackConfig:
         self.envvar_key = self._get_envvar('OS_CLOUD_NAME', 'envvars')
         if self.envvar_key in self.cloud_config['clouds']:
             raise exceptions.ConfigException(
-                '"{0}" defines a cloud named "{1}", but'
-                ' OS_CLOUD_NAME is also set to "{1}". Please rename'
-                ' either your environment based cloud, or one of your'
-                ' file-based clouds.'.format(
-                    self.config_filename, self.envvar_key
-                )
+                f'{self.config_filename!r} defines a cloud named '
+                f'{self.envvar_key!r}, but OS_CLOUD_NAME is also set to '
+                f'{self.envvar_key!r}. '
+                f'Please rename either your environment-based cloud, '
+                f'or one of your file-based clouds.'
             )
 
         self.default_cloud = self._get_envvar('OS_CLOUD')
@@ -297,8 +307,8 @@ class OpenStackConfig:
         self._cache_expiration_time = 0
         self._cache_path = CACHE_PATH
         self._cache_class = 'dogpile.cache.null'
-        self._cache_arguments = {}
-        self._cache_expirations = {}
+        self._cache_arguments: dict[str, ty.Any] = {}
+        self._cache_expirations: dict[str, int] = {}
         self._influxdb_config = {}
         if 'cache' in self.cloud_config:
             cache_settings = _util.normalize_keys(self.cloud_config['cache'])
@@ -408,15 +418,13 @@ class OpenStackConfig:
             ret[newkey] = os.environ[k]
         # If the only environ keys are selectors or behavior modification,
         # don't return anything
-        selectors = set(
-            [
-                'OS_CLOUD',
-                'OS_REGION_NAME',
-                'OS_CLIENT_CONFIG_FILE',
-                'OS_CLIENT_SECURE_FILE',
-                'OS_CLOUD_NAME',
-            ]
-        )
+        selectors = {
+            'OS_CLOUD',
+            'OS_REGION_NAME',
+            'OS_CLIENT_CONFIG_FILE',
+            'OS_CLIENT_SECURE_FILE',
+            'OS_CLOUD_NAME',
+        }
         if set(environkeys) - selectors:
             return ret
         return None
@@ -453,18 +461,37 @@ class OpenStackConfig:
         for path in filelist:
             if os.path.exists(path):
                 try:
-                    with open(path, 'r') as f:
+                    with open(path) as f:
                         if path.endswith('json'):
                             return path, json.load(f)
                         else:
                             return path, yaml.safe_load(f)
-                except IOError as e:
+                except OSError as e:
                     if e.errno == errno.EACCES:
                         # Can't access file so let's continue to the next
                         # file
                         continue
         return (None, {})
 
+    def _validate_config_file(self, path: str, data: ty.Any) -> bool:
+        """Validate config file contains a clouds entry.
+
+        All config files should have a 'clouds' key at a minimum.
+        """
+        if not isinstance(data, dict):
+            raise exceptions.ConfigException(
+                f'Configuration file {path} is empty or not a valid mapping'
+            )
+
+        if 'clouds' not in data:
+            # TODO(stephenfin): This should probably be an error at some point
+            self.log.warning(
+                "Configuration file %s does not contain a 'clouds' key", path
+            )
+            return False
+
+        return True
+
     def _expand_region_name(self, region_name):
         return {'name': region_name, 'values': {}}
 
@@ -477,7 +504,7 @@ class OpenStackConfig:
                     region
                 ):
                     raise exceptions.ConfigException(
-                        'Invalid region entry at: %s' % region
+                        f'Invalid region entry at: {region}'
                     )
                 if 'values' not in region:
                     region['values'] = {}
@@ -514,8 +541,8 @@ class OpenStackConfig:
             return self._expand_regions(regions)
         else:
             # crappit. we don't have a region defined.
-            new_cloud = dict()
-            our_cloud = self.cloud_config['clouds'].get(cloud, dict())
+            new_cloud: dict[str, ty.Any] = {}
+            our_cloud = self.cloud_config['clouds'].get(cloud, {})
             self._expand_vendor_profile(cloud, new_cloud, our_cloud)
             if 'regions' in new_cloud and new_cloud['regions']:
                 return self._expand_regions(new_cloud['regions'])
@@ -540,9 +567,9 @@ class OpenStackConfig:
                 return region
 
         raise exceptions.ConfigException(
-            'Region {region_name} is not a valid region name for cloud'
-            ' {cloud}. Valid choices are {region_list}. Please note that'
-            ' region names are case sensitive.'.format(
+            'Region {region_name} is not a valid region name for cloud '
+            '{cloud}. Valid choices are {region_list}. Please note that '
+            'region names are case sensitive.'.format(
                 region_name=region_name,
                 region_list=','.join([r['name'] for r in regions]),
                 cloud=cloud,
@@ -557,9 +584,7 @@ class OpenStackConfig:
 
         # Only validate cloud name if one was given
         if name and name not in self.cloud_config['clouds']:
-            raise exceptions.ConfigException(
-                "Cloud {name} was not found.".format(name=name)
-            )
+            raise exceptions.ConfigException(f"Cloud {name} was not found.")
 
         our_cloud = self.cloud_config['clouds'].get(name, dict())
         if profile:
@@ -616,16 +641,14 @@ class OpenStackConfig:
                     )
                 elif status == 'shutdown':
                     raise exceptions.ConfigException(
-                        "{profile_name} references a cloud that no longer"
-                        " exists: {message}".format(
-                            profile_name=profile_name, message=message
-                        )
+                        f"{profile_name} references a cloud that no longer "
+                        f"exists: {message}"
                     )
                 _auth_update(cloud, profile_data)
             else:
                 # Can't find the requested vendor config, go about business
                 warnings.warn(
-                    f"Couldn't find the vendor profile {profile_name} for"
+                    f"Couldn't find the vendor profile {profile_name} for "
                     f"the cloud {name}",
                     os_warnings.ConfigurationWarning,
                 )
@@ -643,8 +666,8 @@ class OpenStackConfig:
         for net in networks:
             if value and net[key]:
                 raise exceptions.ConfigException(
-                    "Duplicate network entries for {key}: {net1} and {net2}."
-                    " Only one network can be flagged with {key}".format(
+                    "Duplicate network entries for {key}: {net1} and {net2}. "
+                    "Only one network can be flagged with {key}".format(
                         key=key, net1=value['name'], net2=net['name']
                     )
                 )
@@ -683,9 +706,9 @@ class OpenStackConfig:
             external = key.startswith('external')
             if key in cloud and 'networks' in cloud:
                 raise exceptions.ConfigException(
-                    "Both {key} and networks were specified in the config."
-                    " Please remove {key} from the config and use the network"
-                    " list to configure network behavior.".format(key=key)
+                    f"Both {key} and networks were specified in the config. "
+                    f"Please remove {key} from the config and use the network "
+                    f"list to configure network behavior."
                 )
             if key in cloud:
                 warnings.warn(
@@ -728,10 +751,7 @@ class OpenStackConfig:
             cloud['auth'].pop(target_key, None)
         return cloud
 
-    def _fix_backwards_project(self, cloud):
-        # Do the lists backwards so that project_name is the ultimate winner
-        # Also handle moving domain names into auth so that domain mapping
-        # is easier
+    def _fix_backwards_auth(self, cloud):
         mappings = {
             'domain_id': ('domain_id', 'domain-id'),
             'domain_name': ('domain_name', 'domain-name'),
@@ -743,6 +763,7 @@ class OpenStackConfig:
                 'project-domain-name',
             ),
             'token': ('auth-token', 'auth_token', 'token'),
+            'passcode': ('passcode',),
         }
         if cloud.get('auth_type', None) == 'v2password':
             # If v2password is explcitly requested, this is to deal with old
@@ -776,12 +797,15 @@ class OpenStackConfig:
         for target_key, possible_values in mappings.items():
             target = None
             for key in possible_values:
-                if key in cloud:
-                    target = str(cloud[key])
-                    del cloud[key]
                 if key in cloud['auth']:
                     target = str(cloud['auth'][key])
                     del cloud['auth'][key]
+                # Prefer values NOT from the 'auth' section
+                # as they may contain cli or environment overrides.
+                # See story 2010784 for context.
+                if key in cloud:
+                    target = str(cloud[key])
+                    del cloud[key]
             if target:
                 cloud['auth'][target_key] = target
         return cloud
@@ -881,8 +905,8 @@ class OpenStackConfig:
             options, _args = parser.parse_known_args(argv)
             plugin_names = loading.get_available_plugin_names()
             raise exceptions.ConfigException(
-                "An invalid auth-type was specified: {auth_type}."
-                " Valid choices are: {plugin_names}.".format(
+                "An invalid auth-type was specified: {auth_type}. "
+                "Valid choices are: {plugin_names}.".format(
                     auth_type=options.os_auth_type,
                     plugin_names=",".join(plugin_names),
                 )
@@ -959,8 +983,13 @@ class OpenStackConfig:
                     )
         return clouds
 
-    # TODO(mordred) Backwards compat for OSC transition
-    get_all_clouds = get_all
+    def get_all_clouds(self):
+        warnings.warn(
+            "The 'get_all_clouds' method is a deprecated alias for "
+            "'get_clouds' and will be removed in a future release.",
+            os_warnings.RemovedInSDK60Warning,
+        )
+        return self.get_all()
 
     def _fix_args(self, args=None, argparse=None):
         """Massage the passed-in options
@@ -985,7 +1014,7 @@ class OpenStackConfig:
         os_args = dict()
         new_args = dict()
         for key, val in iter(args.items()):
-            if type(args[key]) == dict:
+            if type(args[key]) is dict:
                 # dive into the auth dict
                 new_args[key] = self._fix_args(args[key])
                 continue
@@ -1034,7 +1063,9 @@ class OpenStackConfig:
             # doing what they want causes them sorrow.
             config['auth_type'] = 'admin_token'
 
-        loader = loading.get_plugin_loader(config['auth_type'])
+        loader: loading.BaseLoader[ty.Any] = loading.get_plugin_loader(
+            config['auth_type']
+        )
 
         # As the name would suggest, v3multifactor uses multiple factors for
         # authentication. As a result, we need to register the configuration
@@ -1047,6 +1078,9 @@ class OpenStackConfig:
         # options in keystoneauth1.loading._plugins.identity.v3.MultiAuth
         # without calling 'load_from_options'.
         if config['auth_type'] == 'v3multifactor':
+            if ty.TYPE_CHECKING:
+                # narrow types
+                assert isinstance(loader, v3_loaders.MultiFactor)
             # We use '.get' since we can't be sure this key is set yet -
             # validation happens later, in _validate_auth
             loader._methods = config.get('auth_methods')
@@ -1150,44 +1184,30 @@ class OpenStackConfig:
                 config['auth'][p_opt.dest] = winning_value
         return config
 
-    def magic_fixes(self, config):
-        """Perform the set of magic argument fixups"""
-
-        # Infer token plugin if a token was given
-        if (
-            ('auth' in config and 'token' in config['auth'])
-            or ('auth_token' in config and config['auth_token'])
-            or ('token' in config and config['token'])
-        ):
-            config.setdefault('token', config.pop('auth_token', None))
-
-        # Infer passcode if it was given separately
-        # This is generally absolutely impractical to require setting passcode
-        # in the clouds.yaml
-        if 'auth' in config and 'passcode' in config:
-            config['auth']['passcode'] = config.pop('passcode', None)
-
-        # These backwards compat values are only set via argparse. If it's
-        # there, it's because it was passed in explicitly, and should win
-        config = self._fix_backwards_api_timeout(config)
-        if 'endpoint_type' in config:
-            config['interface'] = config.pop('endpoint_type')
-
-        config = self._fix_backwards_auth_plugin(config)
-        config = self._fix_backwards_project(config)
-        config = self._fix_backwards_interface(config)
-        config = self._fix_backwards_networks(config)
-        config = self._handle_domain_id(config)
-
+    def _handle_value_types(self, config: dict) -> dict:
         for key in BOOL_KEYS:
             if key in config:
-                if type(config[key]) is not bool:
+                if not isinstance(config[key], bool):
                     config[key] = get_boolean(config[key])
 
         for key in CSV_KEYS:
             if key in config:
                 if isinstance(config[key], str):
                     config[key] = config[key].split(',')
+        return config
+
+    def magic_fixes(self, config):
+        """Perform the set of magic argument fixups"""
+
+        # These backwards compat values are only set via argparse. If it's
+        # there, it's because it was passed in explicitly, and should win
+        config = self._fix_backwards_api_timeout(config)
+        config = self._fix_backwards_auth_plugin(config)
+        config = self._fix_backwards_auth(config)
+        config = self._fix_backwards_interface(config)
+        config = self._fix_backwards_networks(config)
+        config = self._handle_domain_id(config)
+        config = self._handle_value_types(config)
 
         # TODO(mordred): Special casing auth_url here. We should
         #                come back to this betterer later so that it's
@@ -1199,7 +1219,13 @@ class OpenStackConfig:
 
         return config
 
-    def get_one(self, cloud=None, validate=True, argparse=None, **kwargs):
+    def get_one(
+        self,
+        cloud: ty.Optional[str] = None,
+        validate: bool = True,
+        argparse: ty.Optional[argparse_mod.Namespace] = None,
+        **kwargs: ty.Any,
+    ) -> cloud_region.CloudRegion:
         """Retrieve a single CloudRegion and merge additional options
 
         :param string cloud:
@@ -1318,8 +1344,20 @@ class OpenStackConfig:
             influxdb_config=influxdb_config,
         )
 
-    # TODO(mordred) Backwards compat for OSC transition
-    get_one_cloud = get_one
+    def get_one_cloud(
+        self, cloud=None, validate=True, argparse=None, **kwargs
+    ):
+        warnings.warn(
+            "The 'get_one_cloud' method is a deprecated alias for 'get_one' "
+            "and will be removed in a future release.",
+            os_warnings.RemovedInSDK60Warning,
+        )
+        return self.get_one(
+            cloud=cloud,
+            validate=validate,
+            argparse=argparse,
+            **kwargs,
+        )
 
     def get_one_cloud_osc(
         self, cloud=None, validate=True, argparse=None, **kwargs
@@ -1434,7 +1472,7 @@ class OpenStackConfig:
         try:
             with open(config_file) as fh:
                 cur_config = yaml.safe_load(fh)
-        except IOError as e:
+        except OSError as e:
             # Not no such file
             if e.errno != 2:
                 raise
