@@ -341,7 +341,7 @@ class Proxy(proxy.Proxy):
             properties = image_kwargs.pop('properties', {})
             image_kwargs.update(self._make_v2_image_params(meta, properties))
             image_kwargs['name'] = name
-            image = self._create(_image.Image, **image_kwargs)
+            image = self._create(_image.Image, **image_kwargs)  # type: ignore[arg-type]
 
         return image
 
@@ -464,7 +464,7 @@ class Proxy(proxy.Proxy):
         if 'queued' != image.status:
             raise exceptions.SDKException(
                 'Image stage is only possible for images in the queued state. '
-                'Current state is {status}'.format(status=image.status)
+                f'Current state is {image.status}'
             )
 
         if filename:
@@ -601,7 +601,7 @@ class Proxy(proxy.Proxy):
             raise exceptions.SDKException(f"Image creation failed: {str(e)}")
 
     def _make_v2_image_params(self, meta, properties):
-        ret: ty.Dict = {}
+        ret: dict = {}
         for k, v in iter(properties.items()):
             if k in _INT_PROPERTIES:
                 ret[k] = int(v)
@@ -694,9 +694,9 @@ class Proxy(proxy.Proxy):
     ):
         if not self._connection.has_service('object-store'):
             raise exceptions.SDKException(
-                "The cloud {cloud} is configured to use tasks for image "
+                f"The cloud {self._connection.config.name} is configured to use tasks for image "
                 "upload, but no object-store service is available. "
-                "Aborting.".format(cloud=self._connection.config.name)
+                "Aborting."
             )
 
         properties = image_kwargs.get('properties', {})
@@ -759,9 +759,7 @@ class Proxy(proxy.Proxy):
             except exceptions.ResourceFailure as e:
                 glance_task = self.get_task(glance_task)
                 raise exceptions.SDKException(
-                    "Image creation failed: {message}".format(
-                        message=e.message
-                    ),
+                    f"Image creation failed: {e.message}",
                     extra_data=glance_task,
                 )
             finally:
@@ -1028,6 +1026,7 @@ class Proxy(proxy.Proxy):
         member_id = resource.Resource._get_id(member)
         self._delete(
             _member.Member,
+            None,
             member_id=member_id,
             image_id=image_id,
             ignore_missing=ignore_missing,
@@ -1110,6 +1109,7 @@ class Proxy(proxy.Proxy):
         image_id = resource.Resource._get_id(image)
         return self._update(
             _member.Member,
+            None,
             member_id=member_id,
             image_id=image_id,
             **attrs,
@@ -1839,9 +1839,7 @@ class Proxy(proxy.Proxy):
             return task
 
         name = f"{task.__class__.__name__}:{task.id}"
-        msg = "Timeout waiting for {name} to transition to {status}".format(
-            name=name, status=status
-        )
+        msg = f"Timeout waiting for {name} to transition to {status}"
 
         for count in utils.iterate_timeout(
             timeout=wait, message=msg, wait=interval
@@ -1850,9 +1848,7 @@ class Proxy(proxy.Proxy):
 
             if not task:
                 raise exceptions.ResourceFailure(
-                    "{name} went away while waiting for {status}".format(
-                        name=name, status=status
-                    )
+                    f"{name} went away while waiting for {status}"
                 )
 
             new_status = task.status
@@ -1863,12 +1859,10 @@ class Proxy(proxy.Proxy):
                 if task.message == _IMAGE_ERROR_396:
                     task_args = {'input': task.input, 'type': task.type}
                     task = self.create_task(**task_args)
-                    self.log.debug('Got error 396. Recreating task %s' % task)
+                    self.log.debug(f'Got error 396. Recreating task {task}')
                 else:
                     raise exceptions.ResourceFailure(
-                        "{name} transitioned to failure state {status}".format(
-                            name=name, status=new_status
-                        )
+                        f"{name} transitioned to failure state {new_status}"
                     )
 
             self.log.debug(
@@ -1900,21 +1894,68 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_si.Import, requires_id=False)
 
-    # ====== UTILS ======
-    def wait_for_delete(self, res, interval=2, wait=120):
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
         """Wait for a resource to be deleted.
 
         :param res: The resource to wait on to be deleted.
-        :type resource: A :class:`~openstack.resource.Resource` object.
         :param interval: Number of seconds to wait before to consecutive
-            checks. Default to 2.
+            checks.
         :param wait: Maximum number of seconds to wait before the change.
-            Default to 120.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
             to delete failed to occur in the specified seconds.
         """
-        return resource.wait_for_delete(self, res, interval, wait)
+        return resource.wait_for_delete(self, res, interval, wait, callback)
 
     def _get_cleanup_dependencies(self):
         return {'image': {'before': ['identity']}}
