@@ -10,7 +10,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+# This is needed due to https://github.com/eventlet/eventlet/issues/1026 which
+# nova (and possibly others) expose
+from __future__ import annotations
+
+import collections.abc
 import functools
+import logging
+import queue
 import typing as ty
 import urllib
 from urllib.parse import urlparse
@@ -25,87 +32,127 @@ except ImportError:
 import iso8601
 import jmespath
 from keystoneauth1 import adapter
+from keystoneauth1 import session
 
 from openstack import _log
 from openstack import exceptions
 from openstack import resource
+from openstack import utils
 from openstack import warnings as os_warnings
 
+if ty.TYPE_CHECKING:
+    import influxdb as influxdb_client
+    from keystoneauth1 import plugin
+    import prometheus_client
+    import requests
+    from statsd.client import base as statsd_client
 
-ResourceType = ty.TypeVar('ResourceType', bound=resource.Resource)
+    from openstack import connection
 
 
-# The _check_resource decorator is used on Proxy methods to ensure that
-# the `actual` argument is in fact the type of the `expected` argument.
-# It does so under two cases:
-# 1. When strict=False, if and only if `actual` is a Resource instance,
-#    it is checked to see that it's an instance of the `expected` class.
-#    This allows `actual` to be other types, such as strings, when it makes
-#    sense to accept a raw id value.
-# 2. When strict=True, `actual` must be an instance of the `expected` class.
-def _check_resource(strict=False):
-    def wrap(method):
-        def check(self, expected, actual=None, *args, **kwargs):
-            if (
-                strict
-                and actual is not None
-                and not isinstance(actual, resource.Resource)
-            ):
-                raise ValueError(f"A {expected.__name__} must be passed")
-            elif isinstance(actual, resource.Resource) and not isinstance(
-                actual, expected
-            ):
-                raise ValueError(
-                    f"Expected {expected.__name__} but received {actual.__class__.__name__}"
-                )
-
-            return method(self, expected, actual, *args, **kwargs)
-
-        return check
-
-    return wrap
-
-
-def normalize_metric_name(name):
+def normalize_metric_name(name: str) -> str:
     name = name.replace('.', '_')
     name = name.replace(':', '_')
     return name
 
 
+class CleanupDependency(ty.TypedDict):
+    before: list[str]
+    after: list[str]
+
+
 class Proxy(adapter.Adapter):
     """Represents a service."""
 
-    retriable_status_codes: ty.Optional[ty.List[int]] = None
+    retriable_status_codes: list[int] | None = None
     """HTTP status codes that should be retried by default.
 
     The number of retries is defined by the configuration in parameters called
     ``<service-type>_status_code_retries``.
     """
 
-    _resource_registry: ty.Dict[str, ty.Type[resource.Resource]] = {}
+    _resource_registry: dict[str, type[resource.Resource]] = {}
     """Registry of the supported resourses.
 
     Dictionary of resource names (key) types (value).
     """
 
+    _connection: connection.Connection
+
     def __init__(
         self,
-        session,
-        statsd_client=None,
-        statsd_prefix=None,
-        prometheus_counter=None,
-        prometheus_histogram=None,
-        influxdb_config=None,
-        influxdb_client=None,
-        *args,
-        **kwargs,
+        session: session.Session,
+        *,
+        service_type: str | None = None,
+        service_name: str | None = None,
+        interface: str | None = None,
+        region_name: str | None = None,
+        endpoint_override: str | None = None,
+        version: str | None = None,
+        auth: plugin.BaseAuthPlugin | None = None,
+        user_agent: str | None = None,
+        connect_retries: int | None = None,
+        logger: logging.Logger | None = None,
+        allow: dict[str, ty.Any] | None = None,
+        additional_headers: collections.abc.MutableMapping[str, str]
+        | None = None,
+        client_name: str | None = None,
+        client_version: str | None = None,
+        allow_version_hack: bool | None = None,
+        global_request_id: str | None = None,
+        min_version: str | None = None,
+        max_version: str | None = None,
+        default_microversion: str | None = None,
+        status_code_retries: int | None = None,
+        retriable_status_codes: list[int] | None = None,
+        raise_exc: bool | None = None,
+        rate_limit: float | None = None,
+        concurrency: int | None = None,
+        connect_retry_delay: float | None = None,
+        status_code_retry_delay: float | None = None,
+        # everything from here on is SDK-specific
+        statsd_client: statsd_client.StatsClient | None = None,
+        statsd_prefix: str | None = None,
+        prometheus_counter: prometheus_client.Counter | None = None,
+        prometheus_histogram: prometheus_client.Histogram | None = None,
+        influxdb_config: dict[str, ty.Any] | None = None,
+        influxdb_client: influxdb_client.InfluxDBClient | None = None,
     ):
         # NOTE(dtantsur): keystoneauth defaults retriable_status_codes to None,
         # override it with a class-level value.
-        kwargs.setdefault(
-            'retriable_status_codes', self.retriable_status_codes
+        if retriable_status_codes is None:
+            retriable_status_codes = self.retriable_status_codes
+
+        super().__init__(
+            session=session,
+            service_type=service_type,
+            service_name=service_name,
+            interface=interface,
+            region_name=region_name,
+            endpoint_override=endpoint_override,
+            version=version,
+            auth=auth,
+            user_agent=user_agent,
+            connect_retries=connect_retries,
+            logger=logger,
+            allow=allow,
+            additional_headers=additional_headers,
+            client_name=client_name,
+            client_version=client_version,
+            allow_version_hack=allow_version_hack,
+            global_request_id=global_request_id,
+            min_version=min_version,
+            max_version=max_version,
+            default_microversion=default_microversion,
+            status_code_retries=status_code_retries,
+            retriable_status_codes=retriable_status_codes,
+            raise_exc=raise_exc,
+            rate_limit=rate_limit,
+            concurrency=concurrency,
+            connect_retry_delay=connect_retry_delay,
+            status_code_retry_delay=status_code_retry_delay,
         )
-        super().__init__(session=session, *args, **kwargs)
+
         self._statsd_client = statsd_client
         self._statsd_prefix = statsd_prefix
         self._prometheus_counter = prometheus_counter
@@ -118,15 +165,23 @@ class Proxy(adapter.Adapter):
             log_name = 'openstack'
         self.log = _log.setup_logging(log_name)
 
-    def _get_cache_key_prefix(self, url):
+    def _get_cache_key_prefix(self, url: str) -> str:
         """Calculate cache prefix for the url"""
+        if not self.service_type:
+            # narrow type
+            raise RuntimeError('expected service_type to be set')
+
         name_parts = self._extract_name(
             url, self.service_type, self.session.get_project_id()
         )
 
         return '.'.join([self.service_type] + name_parts)
 
-    def _invalidate_cache(self, conn, key_prefix):
+    def _invalidate_cache(
+        self,
+        conn: connection.Connection,
+        key_prefix: str,
+    ) -> None:
         """Invalidate all cache entries starting with given prefix"""
         for k in set(conn._api_cache_keys):
             if k.startswith(key_prefix):
@@ -135,16 +190,20 @@ class Proxy(adapter.Adapter):
 
     def request(
         self,
-        url,
-        method,
-        error_message=None,
-        raise_exc=False,
-        connect_retries=1,
-        global_request_id=None,
-        *args,
-        **kwargs,
-    ):
+        url: str,
+        method: str,
+        error_message: str | None = None,
+        raise_exc: bool = False,
+        connect_retries: int = 1,
+        global_request_id: str | None = None,
+        *args: ty.Any,
+        **kwargs: ty.Any,
+    ) -> requests.Response:
         conn = self._get_connection()
+        if not conn:
+            # narrow type
+            raise RuntimeError('no connection found')
+
         if not global_request_id:
             # Per-request setting should take precedence
             global_request_id = conn._global_request_id
@@ -169,20 +228,21 @@ class Proxy(adapter.Adapter):
                     conn._cache_expirations.get(key_prefix, 0)
                 )
                 # Get from cache or execute and cache
-                response = conn._cache.get_or_create(
+                _response = conn._cache.get_or_create(
                     key=key,
                     creator=super().request,
                     creator_args=(
                         [url, method],
-                        dict(
-                            connect_retries=connect_retries,
-                            raise_exc=raise_exc,
-                            global_request_id=global_request_id,
+                        {
+                            'connect_retries': connect_retries,
+                            'raise_exc': raise_exc,
+                            'global_request_id': global_request_id,
                             **kwargs,
-                        ),
+                        },
                     ),
                     expiration_time=expiration_time,
                 )
+                response = ty.cast('requests.Response', _response)
             else:
                 # invalidate cache if we send modification request or user
                 # asked for cache bypass
@@ -210,7 +270,12 @@ class Proxy(adapter.Adapter):
             raise
 
     @functools.lru_cache(maxsize=256)
-    def _extract_name(self, url, service_type=None, project_id=None):
+    def _extract_name(
+        self,
+        url: str,
+        service_type: str | None = None,
+        project_id: str | None = None,
+    ) -> list[str]:
         """Produce a key name to use in logging/metrics from the URL path.
 
         We want to be able to logic/metric sane general things, so we pull
@@ -286,7 +351,9 @@ class Proxy(adapter.Adapter):
         # Strip out anything that's empty or None
         return [part for part in name_parts if part]
 
-    def _extract_name_consume_url_parts(self, url_parts):
+    def _extract_name_consume_url_parts(
+        self, url_parts: list[str]
+    ) -> list[str]:
         """Pull out every other URL portion.
 
         For example, ``GET /servers/{id}/os-security-groups`` returns
@@ -308,20 +375,41 @@ class Proxy(adapter.Adapter):
 
         return name_parts
 
-    def _report_stats(self, response, url=None, method=None, exc=None):
-        if self._statsd_client:
-            self._report_stats_statsd(response, url, method, exc)
-        if self._prometheus_counter and self._prometheus_histogram:
-            self._report_stats_prometheus(response, url, method, exc)
-        if self._influxdb_client:
-            self._report_stats_influxdb(response, url, method, exc)
+    def _report_stats(
+        self,
+        response: requests.Response | None,
+        url: str | None = None,
+        method: str | None = None,
+        exc: BaseException | None = None,
+    ) -> None:
+        self._report_stats_statsd(response, url, method, exc)
+        self._report_stats_prometheus(response, url, method, exc)
+        self._report_stats_influxdb(response, url, method, exc)
+
+    def _report_stats_statsd(
+        self,
+        response: requests.Response | None,
+        url: str | None = None,
+        method: str | None = None,
+        exc: BaseException | None = None,
+    ) -> None:
+        if not self._statsd_prefix:
+            return None
+
+        if not self._statsd_client:
+            return None
 
-    def _report_stats_statsd(self, response, url=None, method=None, exc=None):
         try:
             if response is not None and not url:
                 url = response.request.url
             if response is not None and not method:
                 method = response.request.method
+
+            # narrow types
+            assert url is not None
+            assert method is not None
+            assert self.service_type is not None
+
             name_parts = [
                 normalize_metric_name(f)
                 for f in self._extract_name(
@@ -352,31 +440,51 @@ class Proxy(adapter.Adapter):
             self.log.exception("Exception reporting metrics")
 
     def _report_stats_prometheus(
-        self, response, url=None, method=None, exc=None
-    ):
+        self,
+        response: requests.Response | None,
+        url: str | None = None,
+        method: str | None = None,
+        exc: BaseException | None = None,
+    ) -> None:
+        if not self._prometheus_counter:
+            return None
+
+        if not self._prometheus_histogram:
+            return None
+
         if response is not None and not url:
             url = response.request.url
         if response is not None and not method:
             method = response.request.method
         parsed_url = urlparse(url)
         endpoint = (
-            f"{parsed_url.scheme}://{parsed_url.netloc}{parsed_url.path}"
+            f"{parsed_url.scheme}://{parsed_url.netloc}{parsed_url.path}"  # type: ignore[str-bytes-safe]
         )
         if response is not None:
-            labels = dict(
-                method=method,
-                endpoint=endpoint,
-                service_type=self.service_type,
-                status_code=response.status_code,
-            )
+            labels = {
+                'method': method,
+                'endpoint': endpoint,
+                'service_type': self.service_type,
+                'status_code': response.status_code,
+            }
             self._prometheus_counter.labels(**labels).inc()
             self._prometheus_histogram.labels(**labels).observe(
                 response.elapsed.total_seconds() * 1000
             )
 
     def _report_stats_influxdb(
-        self, response, url=None, method=None, exc=None
-    ):
+        self,
+        response: requests.Response | None,
+        url: str | None = None,
+        method: str | None = None,
+        exc: BaseException | None = None,
+    ) -> None:
+        if not self._influxdb_client:
+            return None
+
+        if not self._influxdb_config:
+            return None
+
         # NOTE(gtema): status_code is saved both as tag and field to give
         # ability showing it as a value and not only as a legend.
         # However Influx is not ok with having same name in tags and fields,
@@ -385,9 +493,9 @@ class Proxy(adapter.Adapter):
             url = response.request.url
         if response is not None and not method:
             method = response.request.method
-        tags = dict(
-            method=method,
-            name='_'.join(
+        tags = {
+            'method': method,
+            'name': '_'.join(
                 [
                     normalize_metric_name(f)
                     for f in self._extract_name(
@@ -395,8 +503,8 @@ class Proxy(adapter.Adapter):
                     )
                 ]
             ),
-        )
-        fields = dict(attempted=1)
+        }
+        fields = {'attempted': 1}
         if response is not None:
             fields['duration'] = int(response.elapsed.total_seconds() * 1000)
             tags['status_code'] = str(response.status_code)
@@ -418,13 +526,13 @@ class Proxy(adapter.Adapter):
         )
         # Note(gtema) append service name into the measurement name
         measurement = f'{measurement}.{self.service_type}'
-        data = [dict(measurement=measurement, tags=tags, fields=fields)]
+        data = [{'measurement': measurement, 'tags': tags, 'fields': fields}]
         try:
             self._influxdb_client.write_points(data)
         except Exception:
             self.log.exception('Error writing statistics to InfluxDB')
 
-    def _get_connection(self):
+    def _get_connection(self) -> connection.Connection | None:
         """Get the Connection object associated with this Proxy.
 
         When the Session is created, a reference to the Connection is attached
@@ -436,8 +544,11 @@ class Proxy(adapter.Adapter):
         )
 
     def _get_resource(
-        self, resource_type: ty.Type[ResourceType], value, **attrs
-    ) -> ResourceType:
+        self,
+        resource_type: type[resource.ResourceT],
+        value: None | str | resource.ResourceT | utils.Munch,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Get a resource object to work on
 
         :param resource_type: The type of resource to operate on. This should
@@ -463,12 +574,22 @@ class Proxy(adapter.Adapter):
             res = resource_type.new(id=value, connection=conn, **attrs)
         else:
             # An existing resource instance
+            if not isinstance(value, resource_type):
+                raise ValueError(
+                    f'Expected {resource_type.__name__} but received '
+                    f'{value.__class__.__name__}'
+                )
             res = value
             res._update(**attrs)
 
         return res
 
-    def _get_uri_attribute(self, child, parent, name):
+    def _get_uri_attribute(
+        self,
+        child: resource.Resource | str,
+        parent: resource.Resource | str | None,
+        name: str,
+    ) -> str:
         """Get a value to be associated with a URI attribute
 
         `child` will not be None here as it's a required argument
@@ -479,48 +600,51 @@ class Proxy(adapter.Adapter):
         """
         if parent is None:
             value = getattr(child, name)
-        else:
-            value = resource.Resource._get_id(parent)
-        return value
+            assert isinstance(value, str)  # narrow type
+            return value
+
+        return resource.Resource._get_id(parent)
 
     @ty.overload
     def _find(
         self,
-        resource_type: ty.Type[ResourceType],
+        resource_type: type[resource.ResourceT],
         name_or_id: str,
         ignore_missing: ty.Literal[True] = True,
-        **attrs,
-    ) -> ty.Optional[ResourceType]: ...
+        **attrs: ty.Any,
+    ) -> resource.ResourceT | None: ...
 
     @ty.overload
     def _find(
         self,
-        resource_type: ty.Type[ResourceType],
+        resource_type: type[resource.ResourceT],
         name_or_id: str,
         ignore_missing: ty.Literal[False],
-        **attrs,
-    ) -> ResourceType: ...
+        **attrs: ty.Any,
+    ) -> resource.ResourceT: ...
 
     # excuse the duplication here: it's mypy's fault
     # https://github.com/python/mypy/issues/14764
     @ty.overload
     def _find(
         self,
-        resource_type: ty.Type[ResourceType],
+        resource_type: type[resource.ResourceT],
         name_or_id: str,
         ignore_missing: bool,
-        **attrs,
-    ) -> ty.Optional[ResourceType]: ...
+        **attrs: ty.Any,
+    ) -> resource.ResourceT | None: ...
 
     def _find(
         self,
-        resource_type: ty.Type[ResourceType],
+        resource_type: type[resource.ResourceT],
         name_or_id: str,
         ignore_missing: bool = True,
-        **attrs,
-    ) -> ty.Optional[ResourceType]:
+        **attrs: ty.Any,
+    ) -> resource.ResourceT | None:
         """Find a resource
 
+        :param resource_type: The type of resource to find. This should be a
+            :class:`~openstack.resource.Resource` subclass.
         :param name_or_id: The name or ID of a resource to find.
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.NotFoundException` will be
@@ -537,22 +661,22 @@ class Proxy(adapter.Adapter):
             self, name_or_id, ignore_missing=ignore_missing, **attrs
         )
 
-    @_check_resource(strict=False)
     def _delete(
         self,
-        resource_type: ty.Type[ResourceType],
-        value,
-        ignore_missing=True,
-        **attrs,
-    ) -> ty.Optional[ResourceType]:
+        resource_type: type[resource.ResourceT],
+        value: str | resource.ResourceT | None,
+        ignore_missing: bool = True,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT | None:
         """Delete a resource
 
-        :param resource_type: The type of resource to delete. This should
-            be a :class:`~openstack.resource.Resource`
-            subclass with a ``from_id`` method.
-        :param value: The value to delete. Can be either the ID of a
-            resource or a :class:`~openstack.resource.Resource`
-            subclass.
+        :param resource_type: The type of resource to delete. This should be a
+            :class:`~openstack.resource.Resource` subclass.
+        :param value: The resource to delete. This can be the ID of a resource,
+            a :class:`~openstack.resource.Resource` subclass instance, or None
+            for resources that don't have their own identifier or have
+            identifiers with multiple parts. If None, you must pass these other
+            identifiers as kwargs.
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
@@ -579,21 +703,22 @@ class Proxy(adapter.Adapter):
 
         return rv
 
-    @_check_resource(strict=False)
     def _update(
         self,
-        resource_type: ty.Type[ResourceType],
-        value,
-        base_path=None,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        value: str | resource.ResourceT | None,
+        base_path: str | None = None,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Update a resource
 
-        :param resource_type: The type of resource to update.
-        :type resource_type: :class:`~openstack.resource.Resource`
-        :param value: The resource to update. This must either be a
-            :class:`~openstack.resource.Resource` or an id
-            that corresponds to a resource.
+        :param resource_type: The type of resource to update. This should be a
+            :class:`~openstack.resource.Resource` subclass.
+        :param value: The resource to update. This can be the ID of a resource,
+            a :class:`~openstack.resource.Resource` subclass instance, or None
+            for resources that don't have their own identifier or have
+            identifiers with multiple parts. If None, you must pass these other
+            identifiers as kwargs.
         :param str base_path: Base part of the URI for updating resources, if
             different from
             :data:`~openstack.resource.Resource.base_path`.
@@ -612,19 +737,16 @@ class Proxy(adapter.Adapter):
 
     def _create(
         self,
-        resource_type: ty.Type[ResourceType],
-        base_path=None,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        base_path: str | None = None,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Create a resource from attributes
 
-        :param resource_type: The type of resource to create.
-        :type resource_type: :class:`~openstack.resource.Resource`
-        :param str base_path: Base part of the URI for creating resources, if
-            different from
-            :data:`~openstack.resource.Resource.base_path`.
-        :param path_args: A dict containing arguments for forming the request
-            URL, if needed.
+        :param resource_type: The type of resource to create. This should be a
+            :class:`~openstack.resource.Resource` subclass.
+        :param base_path: Base part of the URI for creating resources, if
+            different from :data:`~openstack.resource.Resource.base_path`.
         :param dict attrs: Attributes to be passed onto the
             :meth:`~openstack.resource.Resource.create`
             method to be created. These should correspond
@@ -648,15 +770,15 @@ class Proxy(adapter.Adapter):
 
     def _bulk_create(
         self,
-        resource_type: ty.Type[ResourceType],
-        data,
-        base_path=None,
-    ) -> ty.Generator[ResourceType, None, None]:
+        resource_type: type[resource.ResourceT],
+        data: list[dict[str, ty.Any]],
+        base_path: str | None = None,
+    ) -> ty.Generator[resource.ResourceT, None, None]:
         """Create a resource from attributes
 
-        :param resource_type: The type of resource to create.
-        :type resource_type: :class:`~openstack.resource.Resource`
-        :param list data: List of attributes dicts to be passed onto the
+        :param resource_type: The type of resource to create. This should be a
+            :class:`~openstack.resource.Resource` subclass.
+        :param data: List of attributes dicts to be passed onto the
             :meth:`~openstack.resource.Resource.create`
             method to be created. These should correspond
             to either :class:`~openstack.resource.Body`
@@ -671,29 +793,31 @@ class Proxy(adapter.Adapter):
         """
         return resource_type.bulk_create(self, data, base_path=base_path)
 
-    @_check_resource(strict=False)
     def _get(
         self,
-        resource_type: ty.Type[ResourceType],
-        value=None,
-        requires_id=True,
-        base_path=None,
-        skip_cache=False,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        value: str | resource.ResourceT | None = None,
+        requires_id: bool = True,
+        base_path: str | None = None,
+        skip_cache: bool = False,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Fetch a resource
 
-        :param resource_type: The type of resource to get.
-        :type resource_type: :class:`~openstack.resource.Resource`
-        :param value: The value to get. Can be either the ID of a
-            resource or a :class:`~openstack.resource.Resource`
-            subclass.
-        :param str base_path: Base part of the URI for fetching resources, if
+        :param resource_type: The type of resource to get. This should be a
+            :class:`~openstack.resource.Resource` subclass.
+        :param value: The resource to get. This can be the ID of a resource,
+            a :class:`~openstack.resource.Resource` subclass instance, or None
+            for resources that don't have their own identifier or have
+            identifiers with multiple parts. If None, you must pass these other
+            identifiers as kwargs.
+        :param requires_id: Whether the resource is identified by an ID or not.
+        :param base_path: Base part of the URI for fetching resources, if
             different from
             :data:`~openstack.resource.Resource.base_path`.
-        :param bool skip_cache: A boolean indicating whether optional API
+        :param skip_cache: A boolean indicating whether optional API
             cache should be skipped for this invocation.
-        :param dict attrs: Attributes to be passed onto the
+        :param attrs: Attributes to be passed onto the
             :meth:`~openstack.resource.Resource.get`
             method. These should correspond
             to either :class:`~openstack.resource.Body`
@@ -715,12 +839,12 @@ class Proxy(adapter.Adapter):
 
     def _list(
         self,
-        resource_type: ty.Type[ResourceType],
-        paginated=True,
-        base_path=None,
-        jmespath_filters=None,
-        **attrs,
-    ) -> ty.Generator[ResourceType, None, None]:
+        resource_type: type[resource.ResourceT],
+        paginated: bool = True,
+        base_path: str | None = None,
+        jmespath_filters: str | None = None,
+        **attrs: ty.Any,
+    ) -> ty.Generator[resource.ResourceT, None, None]:
         """List a resource
 
         :param resource_type: The type of resource to list. This should
@@ -759,22 +883,27 @@ class Proxy(adapter.Adapter):
         )
 
         if jmespath_filters and isinstance(jmespath_filters, str):
-            return jmespath.search(jmespath_filters, data)
+            warnings.warn(
+                'Support for jmespath-style filters is deprecated and will be '
+                'removed in a future release.',
+                os_warnings.RemovedInSDK60Warning,
+            )
+            return jmespath.search(jmespath_filters, data)  # type: ignore[no-any-return]
 
         return data
 
     def _head(
         self,
-        resource_type: ty.Type[ResourceType],
-        value=None,
-        base_path=None,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        value: str | resource.ResourceT | None = None,
+        base_path: str | None = None,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Retrieve a resource's header
 
         :param resource_type: The type of resource to retrieve.
         :type resource_type: :class:`~openstack.resource.Resource`
-        :param value: The value of a specific resource to retreive headers
+        :param value: The value of a specific resource to retrieve headers
             for. Can be either the ID of a resource,
             a :class:`~openstack.resource.Resource` subclass,
             or ``None``.
@@ -792,30 +921,50 @@ class Proxy(adapter.Adapter):
         res = self._get_resource(resource_type, value, **attrs)
         return res.head(self, base_path=base_path)
 
-    def _get_cleanup_dependencies(self):
+    def _get_cleanup_dependencies(
+        self,
+    ) -> dict[str, CleanupDependency] | None:
         return None
 
+    # TODO(stephenfin): Add type for filters. We expect the created_at or
+    # updated_at keys
     def _service_cleanup(
         self,
-        dry_run=True,
-        client_status_queue=None,
-        identified_resources=None,
-        filters=None,
-        resource_evaluation_fn=None,
-        skip_resources=None,
-    ):
+        dry_run: bool = True,
+        client_status_queue: queue.Queue[resource.Resource] | None = None,
+        identified_resources: dict[str, resource.Resource] | None = None,
+        filters: dict[str, ty.Any] | None = None,
+        resource_evaluation_fn: ty.Callable[
+            [
+                resource.Resource,
+                dict[str, ty.Any] | None,
+                dict[str, resource.Resource] | None,
+            ],
+            bool,
+        ]
+        | None = None,
+        skip_resources: list[str] | None = None,
+    ) -> None:
         return None
 
     def _service_cleanup_del_res(
         self,
-        del_fn,
-        obj,
-        dry_run=True,
-        client_status_queue=None,
-        identified_resources=None,
-        filters=None,
-        resource_evaluation_fn=None,
-    ):
+        del_fn: ty.Callable[[resource.Resource], None],
+        obj: resource.Resource,
+        dry_run: bool = True,
+        client_status_queue: queue.Queue[resource.Resource] | None = None,
+        identified_resources: dict[str, resource.Resource] | None = None,
+        filters: dict[str, ty.Any] | None = None,
+        resource_evaluation_fn: ty.Callable[
+            [
+                resource.Resource,
+                dict[str, ty.Any] | None,
+                dict[str, resource.Resource] | None,
+            ],
+            bool,
+        ]
+        | None = None,
+    ) -> bool:
         need_delete = False
         try:
             if resource_evaluation_fn and callable(resource_evaluation_fn):
@@ -849,7 +998,11 @@ class Proxy(adapter.Adapter):
             self.log.exception('Cannot delete resource %s: %s', obj, str(e))
         return need_delete
 
-    def _service_cleanup_resource_filters_evaluation(self, obj, filters=None):
+    def _service_cleanup_resource_filters_evaluation(
+        self,
+        obj: resource.Resource,
+        filters: dict[str, ty.Any] | None = None,
+    ) -> bool:
         part_cond = []
         if filters is not None and isinstance(filters, dict):
             for k, v in filters.items():
@@ -881,8 +1034,14 @@ class Proxy(adapter.Adapter):
         else:
             return False
 
-    def should_skip_resource_cleanup(self, resource=None, skip_resources=None):
-        if resource is None or skip_resources is None:
+    def should_skip_resource_cleanup(
+        self, resource: str, skip_resources: list[str] | None = None
+    ) -> bool:
+        if skip_resources is None:
+            return False
+
+        if self.service_type is None:
+            # to keep mypy happy - this should never happen
             return False
 
         resource_name = f"{self.service_type.replace('-', '_')}.{resource}"
@@ -898,7 +1057,10 @@ class Proxy(adapter.Adapter):
 
 # TODO(stephenfin): Remove this and all users. Use of this generally indicates
 # a missing Resource type.
-def _json_response(response, result_key=None, error_message=None):
+def _json_response(
+    response: requests.Response,
+    error_message: str | None = None,
+) -> requests.Response | ty.Any:
     """Temporary method to use to bridge from ShadeAdapter to SDK calls."""
     exceptions.raise_from_response(response, error_message=error_message)
 
@@ -907,11 +1069,11 @@ def _json_response(response, result_key=None, error_message=None):
         return response
 
     # Some REST calls do not return json content. Don't decode it.
-    if 'application/json' not in response.headers.get('Content-Type'):
+    content_type = response.headers.get('Content-Type')
+    if not content_type or 'application/json' not in content_type:
         return response
 
     try:
-        result_json = response.json()
+        return response.json()
     except JSONDecodeError:
         return response
-    return result_json
