@@ -11,6 +11,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
 import warnings
 
 import os_service_types
@@ -44,13 +45,20 @@ class _ServiceDisabledProxyShim:
 
 class ServiceDescription:
     #: Dictionary of supported versions and proxy classes for that version
-    supported_versions = None
+    supported_versions: dict[str, type[proxy_mod.Proxy]] = {}
     #: main service_type to use to find this service in the catalog
-    service_type = None
+    service_type: str
     #: list of aliases this service might be registered as
-    aliases = []
+    aliases: list[str] = []
 
-    def __init__(self, service_type, supported_versions=None, aliases=None):
+    def __init__(
+        self,
+        service_type: str,
+        supported_versions: ty.Optional[
+            dict[str, type[proxy_mod.Proxy]]
+        ] = None,
+        aliases: ty.Optional[list[str]] = None,
+    ):
         """Class describing how to interact with a REST service.
 
         Each service in an OpenStack cloud needs to be found by looking
@@ -67,11 +75,9 @@ class ServiceDescription:
         a service-specific subclass can be used that sets the attributes
         directly.
 
-        :param string service_type:
-            service_type to look for in the keystone catalog
-        :param list aliases:
-            Optional list of aliases, if there is more than one name that might
-            be used to register the service in the catalog.
+        :param service_type: service_type to look for in the keystone catalog
+        :param aliases: Optional list of aliases, if there is more than one
+            name that might be used to register the service in the catalog.
         """
         self.service_type = service_type or self.service_type
         self.supported_versions = (
@@ -223,9 +229,7 @@ class ServiceDescription:
             if not data and instance._strict_proxies:
                 raise exceptions.ServiceDiscoveryException(
                     "Failed to create a working proxy for service "
-                    "{service_type}: No endpoint data found.".format(
-                        service_type=self.service_type
-                    )
+                    f"{self.service_type}: No endpoint data found."
                 )
 
             # If we've gotten here with a proxy object it means we have
@@ -247,15 +251,39 @@ class ServiceDescription:
 
         # Make an adapter to let discovery take over
         version_kwargs = {}
+        supported_versions = sorted([int(f) for f in self.supported_versions])
         if version_string:
             version_kwargs['version'] = version_string
+            if getattr(
+                self.supported_versions[str(supported_versions[0])],
+                'skip_discovery',
+                False,
+            ):
+                # Requested service does not support version discovery
+                # In this case it is more efficient to set the
+                # endpoint_override to the current catalog endpoint value,
+                # otherwise next request will try to perform discovery.
+
+                temp_adapter = config.get_session_client(self.service_type)
+                ep_override = temp_adapter.get_endpoint(skip_discovery=True)
+
+                ep_key = '{service_type}_endpoint_override'.format(
+                    service_type=self.service_type.replace('-', '_')
+                )
+                config.config[ep_key] = ep_override
+
+                return config.get_session_client(
+                    self.service_type,
+                    allow_version_hack=True,
+                    constructor=self.supported_versions[
+                        str(supported_versions[0])
+                    ],
+                    version=version_string,
+                )
         else:
-            supported_versions = sorted(
-                [int(f) for f in self.supported_versions]
-            )
             version_kwargs['min_version'] = str(supported_versions[0])
-            version_kwargs['max_version'] = '{version}.latest'.format(
-                version=str(supported_versions[-1])
+            version_kwargs['max_version'] = (
+                f'{str(supported_versions[-1])}.latest'
             )
 
         temp_adapter = config.get_session_client(
@@ -266,21 +294,15 @@ class ServiceDescription:
             region_name = instance.config.get_region_name(self.service_type)
             if version_kwargs:
                 raise exceptions.NotSupported(
-                    "The {service_type} service for {cloud}:{region_name}"
-                    " exists but does not have any supported versions.".format(
-                        service_type=self.service_type,
-                        cloud=instance.name,
-                        region_name=region_name,
-                    )
+                    f"The {self.service_type} service for "
+                    f"{instance.name}:{region_name} exists but does not have "
+                    f"any supported versions."
                 )
             else:
                 raise exceptions.NotSupported(
-                    "The {service_type} service for {cloud}:{region_name}"
-                    " exists but no version was discoverable.".format(
-                        service_type=self.service_type,
-                        cloud=instance.name,
-                        region_name=region_name,
-                    )
+                    f"The {self.service_type} service for "
+                    f"{instance.name}:{region_name} exists but no version "
+                    f"was discoverable."
                 )
         proxy_class = self.supported_versions.get(str(found_version[0]))
         if proxy_class:
@@ -297,11 +319,9 @@ class ServiceDescription:
         # service catalog that also doesn't have any useful
         # version discovery?
         warnings.warn(
-            "Service {service_type} has no discoverable version. "
+            f"Service {self.service_type} has no discoverable version. "
             "The resulting Proxy object will only have direct "
-            "passthrough REST capabilities.".format(
-                service_type=self.service_type
-            ),
+            "passthrough REST capabilities.",
             category=os_warnings.UnsupportedServiceVersion,
         )
         return temp_adapter
