@@ -69,6 +69,17 @@ VOLUME = {
 
 
 class TestVolume(base.TestCase):
+    def setUp(self):
+        super().setUp()
+        self.resp = mock.Mock()
+        self.resp.body = None
+        self.resp.status_code = 200
+        self.resp.json = mock.Mock(return_value=self.resp.body)
+        self.sess = mock.Mock(spec=adapter.Adapter)
+        self.sess.default_microversion = '3.71'
+        self.sess.post = mock.Mock(return_value=self.resp)
+        self.sess._get_connection = mock.Mock(return_value=self.cloud)
+
     def test_basic(self):
         sot = volume.Volume(VOLUME)
         self.assertEqual("volume", sot.resource_key)
@@ -85,6 +96,7 @@ class TestVolume(base.TestCase):
                 "name": "name",
                 "status": "status",
                 "all_projects": "all_tenants",
+                "user_id": "user_id",
                 "project_id": "project_id",
                 "created_at": "created_at",
                 "updated_at": "updated_at",
@@ -141,36 +153,45 @@ class TestVolume(base.TestCase):
             VOLUME["OS-SCH-HNT:scheduler_hints"], sot.scheduler_hints
         )
 
-
-class TestVolumeActions(TestVolume):
-    def setUp(self):
-        super().setUp()
-        self.resp = mock.Mock()
-        self.resp.body = None
-        self.resp.status_code = 200
-        self.resp.json = mock.Mock(return_value=self.resp.body)
-        self.sess = mock.Mock(spec=adapter.Adapter)
-        self.sess.default_microversion = '3.60'
-        self.sess.post = mock.Mock(return_value=self.resp)
-        self.sess._get_connection = mock.Mock(return_value=self.cloud)
-
     def test_extend(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.extend(self.sess, '20'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {"os-extend": {"new_size": "20"}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
 
+    def test_complete_extend(self):
+        sot = volume.Volume(**VOLUME)
+
+        self.assertIsNone(sot.complete_extend(self.sess))
+
+        url = f'volumes/{FAKE_ID}/action'
+        body = {'os-extend_volume_completion': {'error': False}}
+        self.sess.post.assert_called_with(
+            url, json=body, microversion=sot._max_microversion
+        )
+
+    def test_complete_extend_error(self):
+        sot = volume.Volume(**VOLUME)
+
+        self.assertIsNone(sot.complete_extend(self.sess, error=True))
+
+        url = f'volumes/{FAKE_ID}/action'
+        body = {'os-extend_volume_completion': {'error': True}}
+        self.sess.post.assert_called_with(
+            url, json=body, microversion=sot._max_microversion
+        )
+
     def test_set_volume_readonly(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.set_readonly(self.sess, True))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-update_readonly_flag': {'readonly': True}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -181,7 +202,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.set_readonly(self.sess, False))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-update_readonly_flag': {'readonly': False}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -192,7 +213,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.set_bootable_status(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-set_bootable': {'bootable': True}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -203,7 +224,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.set_bootable_status(self.sess, False))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-set_bootable': {'bootable': False}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -214,7 +235,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.set_image_metadata(self.sess, {'foo': 'bar'}))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-set_image_metadata': {'foo': 'bar'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -230,7 +251,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.delete_image_metadata(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body_a = {'os-unset_image_metadata': 'foo'}
         body_b = {'os-unset_image_metadata': 'baz'}
         self.sess.post.assert_has_calls(
@@ -249,7 +270,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.delete_image_metadata_item(self.sess, 'foo'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-unset_image_metadata': 'foo'}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -260,7 +281,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.reset_status(self.sess, '1', '2', '3'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-reset_status': {
                 'status': '1',
@@ -277,7 +298,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.reset_status(self.sess, status='1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-reset_status': {
                 'status': '1',
@@ -309,7 +330,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.revert_to_snapshot(self.sess, '1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'revert': {'snapshot_id': '1'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -321,7 +342,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.attach(self.sess, '1', instance='2'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-attach': {'mountpoint': '1', 'instance_uuid': '2'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -332,7 +353,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.attach(self.sess, '1', host_name='2'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-attach': {'mountpoint': '1', 'host_name': '2'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -348,7 +369,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.detach(self.sess, '1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-detach': {'attachment_id': '1'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -361,7 +382,7 @@ class TestVolumeActions(TestVolume):
             sot.detach(self.sess, '1', force=True, connector={'a': 'b'})
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-force_detach': {'attachment_id': '1', 'connector': {'a': 'b'}}
         }
@@ -374,7 +395,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.unmanage(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-unmanage': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -385,7 +406,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.retype(self.sess, '1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-retype': {'new_type': '1'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -396,7 +417,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.retype(self.sess, '1', migration_policy='2'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-retype': {'new_type': '1', 'migration_policy': '2'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -407,7 +428,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.migrate(self.sess, host='1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-migrate_volume': {'host': '1'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -422,7 +443,7 @@ class TestVolumeActions(TestVolume):
             )
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-migrate_volume': {
                 'host': '1',
@@ -448,7 +469,7 @@ class TestVolumeActions(TestVolume):
             )
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-migrate_volume': {
                 'cluster': '1',
@@ -466,7 +487,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.complete_migration(self.sess, new_volume_id='1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-migrate_volume_completion': {'new_volume': '1', 'error': False}
         }
@@ -481,7 +502,7 @@ class TestVolumeActions(TestVolume):
             sot.complete_migration(self.sess, new_volume_id='1', error=True)
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-migrate_volume_completion': {'new_volume': '1', 'error': True}
         }
@@ -494,7 +515,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.force_delete(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-force_delete': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -511,7 +532,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertDictEqual({'a': 'b'}, sot.upload_to_image(self.sess, '1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-volume_upload_image': {'image_name': '1', 'force': False}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -543,7 +564,7 @@ class TestVolumeActions(TestVolume):
             ),
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-volume_upload_image': {
                 'image_name': '1',
@@ -564,7 +585,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.reserve(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-reserve': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -575,7 +596,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.unreserve(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-unreserve': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -586,7 +607,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.begin_detaching(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-begin_detaching': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -597,7 +618,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.abort_detaching(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-roll_detaching': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -615,7 +636,7 @@ class TestVolumeActions(TestVolume):
             {'c': 'd'}, sot.init_attachment(self.sess, {'a': 'b'})
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-initialize_connection': {'connector': {'a': 'b'}}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -626,7 +647,7 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.terminate_attachment(self.sess, {'a': 'b'}))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-terminate_connection': {'connector': {'a': 'b'}}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -662,7 +683,7 @@ class TestVolumeActions(TestVolume):
         self.sess.post.assert_called_with(
             url,
             json=body,
-            microversion='3.60',
+            microversion=sot._max_microversion,
             headers={},
             params={},
         )
@@ -734,6 +755,6 @@ class TestVolumeActions(TestVolume):
         self.sess.default_microversion = '3.50'
         self.assertIsNone(sot.extend(self.sess, '20'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {"os-extend": {"new_size": "20"}}
         self.sess.post.assert_called_with(url, json=body, microversion="3.50")
